/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.engine3d.util;

import net.jcip.annotations.Immutable;

/**
 * Simple class to create visualization filter object.
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
@Immutable
public final class VoxelFilter {
    
    /**
     * Filter index.
     */
    private final int index;
    
    /**
     * The property name to filter.
     */
    private final String property;
    
    /**
     * The filtered value.
     */
    private final float value;
    
    /**
     * The filtering type.
     */
    private final VoxelFilteringType filteringType;
    
    /**
     * If this filter will hide the NaN values.
     */
    private final boolean hideNaN;

    /**
     * Create a filter with the filtered value and the filtering type.
     * @param index
     * @param property The property name.
     * @param value The value to filter.
     * @param filteringType
     * @param hideNaN If this filter will hide the NaN values.
     */
    public VoxelFilter(final int index, final String property, final float value, 
            final VoxelFilteringType filteringType, final boolean hideNaN) {
        this.index = index;
        this.property = property;
        this.value = value;
        this.filteringType = filteringType;
        this.hideNaN = hideNaN;
    }

    /**
     * Get the filter index.
     * @return 
     */
    public final int getIndex() {
        return index;
    }
    
    /**
     * Get the property name.
     * @return 
     */
    public final String getProperty() {
        return property;
    }

    /**
     * Get the filtered value.
     * @return 
     */
    public final float getValue() {
        return value;
    }
    
    /**
     * Get the filtering type.
     * @return 
     */
    public final VoxelFilteringType getFilteringType() {
        return filteringType;
    }

    /**
     * Get if this filter will hide the NaN values.
     * @return 
     */
    public boolean isHideNaN() {
        return hideNaN;
    }
    
    /**
     * Check if the specified value is between the min and max value.
     * @param value
     * @return False if between or False.
     */
    public final boolean isHiden(final float value) {        
        /* Hide NaN values */
        if (hideNaN && Float.isNaN(value)) {
            return true;
        }
        
        /* Hide voxel */
        switch(filteringType) {
            case NOTEQUAL: return value != this.value;
            case LESSER: return value < this.value;
            case EQUAL: return value == this.value;
            case GREATER: return value > this.value;
        }
        
        return false;
    }
    
}
